#include "util.h"

#ifndef lint
char copyright[] =
"ANALYSIS - Version "VERSION"\nCopyright (c) 1995 Brian A. Lantz. All rights reserved.\n";
#endif /* not lint */


char *getargp;

const char *classstr (int which);
char *getarg (char *line);



const char *
classstr (which)
int which;
{
	switch (which)	{
		case TYPE_OP_NOVICE:		return ("Novice");
		case TYPE_OP_TECHNICIAN:	return ("Tech");
		case TYPE_OP_TECHNICIANPLUS:	return ("Tech+");
		case TYPE_OP_GENERAL:		return ("Genrl");
		case TYPE_OP_ADVANCED:		return ("Advan");
		case TYPE_OP_EXTRA:		return ("Extra");
		case TYPE_OP_UNKNOWN:
		default:			return ("????");
	}
}


char *getarg(line)
char *line;
{
char *arg;

	if (line)
		getargp = line;
	arg = getargp;
	while (*getargp && *getargp != '|') {
		getargp++;
	}
	if (*getargp)
		*getargp++ = '\0';
	return arg;
}


int
main (int argc,char *argv[])
{
long total = 0, listed = 0, entry;
FILE *fp;
GDBM_FILE db;
datum key_data, return_data;
struct appl_dat ap, ap2;
char buffer[1024];
char callsign[2048];
char type_purpose[8], type_applicant;
char *cp;
char const *desc = NULLCHAR;
char mode = 0;
int skipped = 0;


	if (argc > 1)
		mode = 1;
	if ((fp = fopen(_PATH_APPL_FILE, "r")) == NULLFILE)	{
		printf ("Error #%d opening file %s\n", errno, _PATH_APPL_FILE);
		return 0;
	}
	db = gdbm_open (_PATH_DATABASE, 0, GDBM_READER, 00664, NULL);
	if (db == NULL)	{
		printf ("Error setting up gdbm file: %s\n", gdbm_strerror(gdbm_errno));
		return 0;
	}
	puts ("Summary of FCC important licensing changes in the last 24 hours");
	puts ("Only new licenses, upgrades, and callsign changes are listed here\n");
	if (mode)
		printf ("Output limited by state: %s\n\n", argv[1]);
	while (!feof(fp))	{
		fgets (buffer, 1024, fp);
		if (feof(fp))
			continue;
		getargp = buffer;
		strcpy (callsign, getarg(0));

		total++;
		/* discard id_file_num */
		(void) getarg(0);
		/* look at but not store the type_purpose */
		strcpy (type_purpose, getarg(0));
		/* look at but not store the type_applicant */
		type_applicant = *(getarg(0));
		if (type_applicant != TYPE_APP_INDIVIDUAL)
			continue;
		memset (ap.name_licensee, ' ', 40);
		strcpy (ap.name_licensee, getarg(0));
		memset (ap.text_street, ' ', 35);
		strcpy (ap.text_street, getarg(0));
		memset (ap.text_pobox, ' ', 20);
		strcpy (ap.text_pobox, getarg(0));
		if (!ap.text_street[0] && ap.text_pobox[0])
			sprintf (ap.text_street, "P.O. Box %s", ap.text_pobox);
		memset (ap.city, ' ', 20);
		strcpy (ap.city, getarg(0));
		memset (ap.state, ' ', 2);
		strcpy (ap.state, getarg(0));
		memset (ap.zipcode, ' ', 9);
		strcpy (ap.zipcode, getarg(0));
		ap.zipcode[6] = 0;
#if 0	/* discard these times - unused here */
		ap.date_of_birth = parsetime(getarg(0));
		ap.date_issue = parsetime(getarg(0));
		ap.date_expire = parsetime(getarg(0));
		ap.date_last_change = parsetime(getarg(0));
#endif
		/* discard the id_examiner, flag_renewal_notice, and date_renewal_notice */

		if (mode && strncasecmp (ap.state, argv[1], 2))
			continue;

		key_data.dptr = callsign;
		key_data.dsize = strlen(callsign) + 1;

		return_data = gdbm_fetch (db, key_data);
		if (return_data.dptr != NULL)	{
			(void) unsqueeze (&ap2, return_data.dptr);
			free (return_data.dptr);
		} else	{
			ap2.type_op_class = TYPE_OP_UNKNOWN;
			memset (ap2.callsign_prev, ' ', 10);
			ap2.callsign_prev[0] = 0;
			ap2.type_prev_op_class = TYPE_OP_UNKNOWN;
		}
		cp = type_purpose;
		entry = 0;
		while (*cp)	{
			skipped = 0;
			switch (*cp++)	{
				case TYPE_PUR_NEW:
					desc = "New Licensee";
					break;
				case TYPE_PUR_CLASSCHANGE:
					desc = "Upgrade";
					if (*cp == TYPE_PUR_CALLCHANGE)	{
						cp++;
						desc = "Upgrade/Call change";
					}
					break;
				case TYPE_PUR_NAMECHANGE:
					desc = "Name change";
					skipped = 1;
					break;
				case TYPE_PUR_ADDRCHANGE:
					desc = "Address change";
					skipped = 1;
					break;
				case TYPE_PUR_CALLCHANGE:
					desc = "Callsign change";
					break;
				case TYPE_PUR_RENEWAL:
				case TYPE_PUR_RENEWALR:
				case TYPE_PUR_RENEWALB:
					desc = "Renewal";
					skipped = 1;
					break;
				case TYPE_PUR_SPECIAL:
					desc = "Special Change";
					skipped = 1;
					break;
				case TYPE_PUR_MYSTERY:
				case TYPE_PUR_UNKNOWN:
				case TYPE_PUR_ADDRECORD:
				case TYPE_PUR_DUPREQUEST:
				case TYPE_PUR_ISSUECHANGE:
				case TYPE_PUR_SUPERCEDE:
				case TYPE_PUR_CORRECTION:
				case TYPE_PUR_DELETE:
				case TYPE_PUR_MODIFYB:
				case TYPE_PUR_RESTOREBOTH:
				case TYPE_PUR_RESTORE:
					skipped = 1;
					break;
			}
			if (!skipped)	{
				if (!entry)	{
					listed++;
					if (!mode)
						printf ("%-4.2s%-8.6s%-s\n    (%-s",
							ap.state, callsign, ap.name_licensee, desc);
					else
						printf ("%-8.6s%-42.40s%-21.21s%-s\n        %-37.35s%-22.20s%-4.2s%s\n",
							callsign, ap.name_licensee, desc, classstr(ap2.type_op_class),
							ap.text_street, ap.city, ap.state, ap.zipcode);
				} else	{
					if (mode && entry == 1)
						printf ("        (also: ");
					else
						printf (", ");
					printf ("%s", desc);
				}
				entry++;
			}
		}
		if ((!mode && entry) || (mode && entry > 1))
			printf (")\n");
		if (mode && entry)
			printf ("\n");
	}
	printf ("\nThe FCC processed %ld applications - of which %ld are listed here\n", total, listed);
	return (0);
}
